Attribute VB_Name = "TCTTools"
Option Explicit


Type TDFProperties
    File As String      'Full path
    Vg1Start As Single
    Vg1Stop As Single
    Vg1Pitch As Single
    Vg1Steps As Integer
    Vg2Start As Single
    Vg2Stop As Single
    Vg2Pitch As Single
    Vg2Steps As Integer
    VaMax As Single
    IaMax As Single
    Vg2Max As Single
    Ig2Max As Single
    Tb() As Single
    Loaded As Boolean
End Type

Public Const VACOL = 0, IACOL = 1, VG1COL = 2, IG1COL = 3, VG2COL = 4, IG2COL = 5

Function GetTDFProperties(Properties As TDFProperties) As Boolean
'Fill up the TDFProperties structure.
'File member must be initialized
'Returns True if succeeds

Dim S As String
Dim F As Integer
Dim T As String
Dim I As Integer
Dim J As Integer
Dim CurVg1 As Single
Dim CurVg2 As Single
Dim TempVal As Single

    F = IsOpen(Properties.File, "R", 10)
    If F <> 0 Then
        S = Input(LOF(F), F)
        XClose F, 10
    Else
        Exit Function   'Error
    End If
    With Properties
    .Loaded = False
    I = 1
    J = 1
    T = Substring(S, I, vbCrLf)
    If Len(T) Then  'If the file is not empty
        CurVg1 = Val(Substring(T, 3, " "))
        .Vg1Start = CurVg1
        CurVg2 = Val(Substring(T, 5, " "))
        .Vg2Start = CurVg2
    Else
        Exit Function   'Error
    End If
    .Vg1Pitch = 0
    .Vg2Pitch = 0
    .Vg1Steps = 1
    .Vg2Steps = 1
    Do While Len(T)     'up to end of file
        TempVal = Val(Substring(T, 3, " "))
        If TempVal <> CurVg1 Then 'Another grid1 serie
            .Vg1Pitch = CurVg1 - TempVal
            CurVg1 = TempVal
'            If .Vg1Pitch = 0 Then .Vg1Pitch = CurVg1
            .Vg1Steps = .Vg1Steps + 1
            TempVal = Val(Substring(T, 5, " "))
            If TempVal <> CurVg2 Then 'Another grid2 serie
                CurVg2 = TempVal
                If .Vg2Pitch = 0 Then .Vg2Pitch = CurVg2
                .Vg2Steps = .Vg2Steps + 1
            End If
        End If
        TempVal = Val(Substring(T, 2, " "))
        If TempVal > .IaMax Then .IaMax = TempVal
        TempVal = Val(Substring(T, 1, " "))
        If TempVal > .VaMax Then .VaMax = TempVal
        TempVal = Val(Substring(T, 6, " "))
        If TempVal > .Ig2Max Then .Ig2Max = TempVal
        TempVal = Val(Substring(T, 5, " "))
        If TempVal > .Vg2Max Then .Vg2Max = TempVal
        ReDim Preserve .Tb(0 To 5, 0 To I - 1)
'        Debug.Print UBound(.Tb, 1), UBound(.Tb, 2)
        For F = 0 To 5
            .Tb(F, I - 1) = Val(Substring(T, F + 1, " "))
'            Debug.Print .Tb(F, I - 1),
        Next F
        I = I + 1
        T = Substring(S, I, vbCrLf)
    Loop
    T = Substring(S, I - 1, vbCrLf) 'Last line
    .Vg1Stop = Val(Substring(T, 3, " "))
    .Vg2Stop = Val(Substring(T, 5, " "))
    .Loaded = True
    End With
    GetTDFProperties = True
End Function

Function SelectRecords(TbIn() As Single, Column As Integer, Value As Single, Tolerance As Single, TbOut() As Single) As Integer
'Fill TbOut with matching records found in TbIn
'Returns the number of matches.

Dim I As Integer
Dim CC As Integer
Dim C As Integer
Dim Found As Integer
Dim V As Single

    CC = UBound(TbIn, 1)
    If CC < Column Then Exit Function  'not that column
    ReDim TbOut(0 To CC, 0 To Found)
    Tolerance = Tolerance + 1
    For I = 0 To UBound(TbIn, 2)
        V = TbIn(Column, I)
        If Value = V Or ((Value * Tolerance) > V And (Value / Tolerance) < V) Then   'Match
            ReDim Preserve TbOut(0 To CC, 0 To Found)
            For C = 0 To CC
                TbOut(C, Found) = TbIn(C, I)
            Next C
            Found = Found + 1
        End If
    Next I
    SelectRecords = Found
End Function

Function NearRecord(TbIn() As Single, Column As Integer, Value As Single, TbOut() As Single) As Integer
'Find the nearest matching record.
'Returns index where found in TbIn

Dim I As Integer
Dim J As Integer
Dim F As Integer
Dim Delta As Single

    ReDim TbOut(5, 0)
    Delta = 3.402823E+38    'Upper limit
    For I = 0 To UBound(TbIn, 2)
        If Abs(Value - TbIn(Column, I)) < Abs(Delta) Then
            Delta = Abs(Value - TbIn(Column, I))
            F = I
        End If
    Next I
    Delta = Delta / (1 + Value)    'Voids division by 0
    For I = 0 To 5
        TbOut(I, 0) = TbIn(I, F)
    Next I
    NearRecord = F
End Function

Function BetweenRecords(TbIn() As Single, StartAtRecord As Integer, Column As Integer, Value As Single, TbOut() As Single) As Integer
'Returns a single record whith values interpolated if not exact match found.
'Returns the record's (line) number where the match occured, 0 if not found.

Dim TbTemp() As Single
Dim I As Integer
Dim C As Integer
Dim V As Single
Dim Step As Single
Dim K As Single

    TbTemp = TbIn    'Preserve TbIn
    For I = StartAtRecord To UBound(TbTemp, 2)
        V = TbTemp(Column, I)
        If V >= Value Then
            BetweenRecords = I
            ReDim TbOut(5, 0)   'Setup output
            For C = 0 To 5
                TbOut(C, 0) = TbTemp(C, I)
            Next C
            If V = Value Then Exit Function  'Exact match or first ! Done.
            If I > 0 Then   'Interpolate with previous record
            Step = TbOut(Column, 0) - TbTemp(Column, I - 1)
            K = V - Value
            K = Step / K
                For C = 0 To 5
'                    TbOut(C, 0) = (TbOut(C, 0) + TbTemp(C, I - 1)) / 2
                    If C <> VG1COL And K > 0 Then TbOut(C, 0) = TbOut(C, 0) - (TbOut(C, 0) - TbTemp(C, I - 1)) / K
                Next C
                Exit Function
            End If
        End If
    Next I
    BetweenRecords = 0  'Not found
End Function

Sub RpMuS(TDF As String, Va As Single, Ia As Single, Rp As Single, Mu As Single, Slope As Single)
'Returns basic parameters at this working point

Dim ThisTDF As TDFProperties
Dim TempTb1() As Single
Dim TempTb2() As Single
Dim TempTb3() As Single
Dim I As Integer
Dim C As Integer
Dim CurIa As Single
Dim CurVa As Single
Dim Vg1 As Single
Dim D As Integer

    ThisTDF.File = TDF
    If GetTDFProperties(ThisTDF) = False Then Exit Sub
        
    I = SelectRecords(ThisTDF.Tb, IACOL, Ia, 0.1, TempTb1)  'All records with this anode current
    D = NearRecord(TempTb1, VACOL, Va, TempTb2)             'The nearest with this anode voltage
    Debug.Print "dVa= " & D,
    D = BetweenRecords(TempTb1, D - 1, VACOL, Va, TempTb2)
    CurIa = TempTb2(IACOL, 0)
    CurVa = TempTb2(VACOL, 0)
    Base.MoveMarker CurIa, CurVa
    Vg1 = TempTb2(VG1COL, 0)                                'Current Vg1
    If TempTb2(IACOL, 0) < Ia Then
        Vg1 = Vg1 + ThisTDF.Vg1Pitch                        'Choose the best
    Else
        Vg1 = Vg1 - ThisTDF.Vg1Pitch
    End If
    I = SelectRecords(ThisTDF.Tb, VG1COL, Vg1, 0, TempTb1)  'All records with this Vg1.
    D = NearRecord(TempTb1, IACOL, CurIa, TempTb2)
    Debug.Print "dIa= " & D,
    Base.MoveMarker TempTb2(IACOL, 0), TempTb2(VACOL, 0)
    Mu = Abs((CurVa - TempTb2(VACOL, 0)) / ThisTDF.Vg1Pitch)
    D = NearRecord(TempTb1, VACOL, CurVa, TempTb2)          'The nearest for the same voltage
    Debug.Print "dVa1=" & D
    Base.MoveMarker TempTb2(IACOL, 0), TempTb2(VACOL, 0)
    Slope = Abs((CurIa - TempTb2(IACOL, 0)) / ThisTDF.Vg1Pitch)
    Rp = Mu / Slope / 1000
End Sub

Function GetVg1Serie(TDF As String, Ia As Single, Va As Single, Record() As Single) As Single
'Returns the Vg1 serie and the nearest matching record

Static ThisTDF As TDFProperties
Dim TempTb() As Single
Dim I As Integer

    If TDF <> ThisTDF.File Then
        ThisTDF.File = TDF
        If GetTDFProperties(ThisTDF) = False Then Exit Function
    End If
    I = SelectRecords(ThisTDF.Tb, IACOL, Ia, 0.05, TempTb)
    I = NearRecord(TempTb, VACOL, Va, Record)
    GetVg1Serie = Record(VG1COL, 0)
End Function

